unit Xtreg;

interface

uses Classes, SysUtils, DsgnIntf, Consts;

procedure Register;

implementation

uses Db, DbTables, DbNextNo, DSDesign, TypInfo,
     Wave,    WavEdit,
     PrnTest, PrnEdit;


{ TDBStringProperty }

type
  TDBStringProperty = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValueList(List: TStrings); virtual; abstract;
    procedure GetValues(Proc: TGetStrProc); override;
  end;

function TDBStringProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList, paSortList, paMultiSelect];
end;

procedure TDBStringProperty.GetValues(Proc: TGetStrProc);
var
  I: Integer;
  Values: TStringList;
begin
  Values := TStringList.Create;
  try
    GetValueList(Values);
    for I := 0 to Values.Count - 1 do Proc(Values[I]);
  finally
    Values.Free;
  end;
end;

{ TDataFieldProperty }

type
  TDataFieldProperty = class(TDBStringProperty)
  public
    function GetDataSourcePropName: string; virtual;
    procedure GetValueList(List: TStrings); override;
  end;

function TDataFieldProperty.GetDataSourcePropName: string;
begin
  Result := 'DataSource';
end;

procedure TDataFieldProperty.GetValueList(List: TStrings);
var
  Instance: TComponent;
  PropInfo: PPropInfo;
  DataSource: TDataSource;
begin
  Instance := GetComponent(0);
  PropInfo := TypInfo.GetPropInfo(Instance.ClassInfo, GetDataSourcePropName);
  if (PropInfo <> nil) and (PropInfo^.PropType^.Kind = tkClass) then
  begin
    DataSource := TObject(GetOrdProp(Instance, PropInfo)) as TDataSource;
    if (DataSource <> nil) and (DataSource.DataSet <> nil) then
      DataSource.DataSet.GetFieldNames(List);
  end;
end;

{ TNextNoFieldProperty }

type
  TNextNoFieldProperty = class(TDataFieldProperty)
  public
    function GetDataSourcePropName: string; override;
  end;

function TNextNoFieldProperty.GetDataSourcePropName: string;
begin
  Result := 'NextNoSource';
end;

procedure Register;
begin
  RegisterPropertyEditor(TypeInfo(string), TDbNextNo, 'KeyField', TNextNoFieldProperty);
  RegisterPropertyEditor(TypeInfo(string), TDbNextNo, 'NoField', TNextNoFieldProperty);

  RegisterComponentEditor(TWave, TWaveEditor);
  RegisterPropertyEditor(TypeInfo(TWaveform), nil, '', TWaveformProperty);

  RegisterComponentEditor(TPrintout, TSectionEditor);
  RegisterPropertyEditor(TypeInfo(TStrings), TPrintout, 'Sections', TSectionProperty);
end;

end.
